package com.qihoo360.mobilesafe.reward.presenter

import com.qihoo360.apullsdk.apull.protocol.model.ApullTemplateBase
import com.qihoo360.mobilesafe.reward.RewardConstant.Companion.PAGE_REWARD_HOUR_SUBSCENE_1
import com.qihoo360.mobilesafe.reward.RewardConstant.Companion.PAGE_REWARD_HOUR_SUBSCENE_4
import com.qihoo360.mobilesafe.reward.RewardConstant.Companion.PAGE_REWARD_SCENE
import com.qihoo360.mobilesafe.reward.data.ApullItemWrapper
import com.qihoo360.mobilesafe.reward.data.RewardUtils
import com.qihoo360.mobilesafe.reward.respository.RewardTryResponse
import com.qihoo360.newssdk.protocol.model.TemplateBase
import java.util.HashMap

class RewardViewModel {

     /**
      * 所有展示的下载红包模版（已过滤）
      * 下载类：key=DownloadId
      * 打开类：key=uniqueId
      */
     private var mAllRewardTemplates = mutableMapOf<String,ApullItemWrapper?>()

    /**
      * 正在抢红包的红包模版key = DownloadId或uniqueId
      */
     private var mWorkingTemplates = mutableMapOf<String,ApullItemWrapper?>()


    /**
     * 红包流程的缓存，try,commit等，key = uniqueId
     */
     private var mRewardInfos = mutableMapOf<String,RewardUtils.RewardInfo?>()

    /**
     *Try请求缓存结果
     */
    private val mTryResponseMap = mutableMapOf<String,RewardTryResponse?>()

    /**
     * 红包返回弹窗模板
     */
    private val mTwiceDialogTemplates = mutableListOf<ApullTemplateBase?>()

    /**
     *整点红包模版
     */
    private val mRewardHourTemplates = HashMap<String, ApullItemWrapper?>()


     fun cacheRewardTemplate(templateBase: ApullTemplateBase?){
         templateBase?.let {
             val apullItemWrapper = ApullItemWrapper(it)
             apullItemWrapper.getDownloadId()?.apply {
                 mAllRewardTemplates[this] = apullItemWrapper
             }
             if(apullItemWrapper.getProtocolType() == ApullItemWrapper.MV){
                 if(apullItemWrapper.uniqueId.isNotEmpty()) {
                     mAllRewardTemplates[apullItemWrapper.uniqueId] = apullItemWrapper
                 }
             }
             //整点红包存在查看类
             if(templateBase.scene ==  PAGE_REWARD_SCENE && (templateBase.subscene in PAGE_REWARD_HOUR_SUBSCENE_1..PAGE_REWARD_HOUR_SUBSCENE_4)){
                 if(apullItemWrapper.uniqueId.isNotEmpty()) {
                     mAllRewardTemplates[apullItemWrapper.uniqueId] = apullItemWrapper
                 }
             }
         }
     }

     fun getRewardTemplate(key:String): ApullTemplateBase?{
          return mAllRewardTemplates[key]?.getTemplate()
     }

     fun cacheWorkingTemplate(templateBase: ApullTemplateBase?){

         templateBase?.let {
             val apullItemWrapper = ApullItemWrapper(it)
             apullItemWrapper.getDownloadId()?.apply {
                 mWorkingTemplates[this] = apullItemWrapper
             }

             if(apullItemWrapper.getProtocolType() == ApullItemWrapper.MV){
                 if(apullItemWrapper.uniqueId.isNotEmpty()){
                     mWorkingTemplates[apullItemWrapper.uniqueId] = apullItemWrapper
                 }
             }

             //整点红包存在查看类
             if(templateBase.scene == PAGE_REWARD_SCENE && (templateBase.subscene in PAGE_REWARD_HOUR_SUBSCENE_1..PAGE_REWARD_HOUR_SUBSCENE_4)){
                 if(apullItemWrapper.uniqueId.isNotEmpty()) {
                     mWorkingTemplates[apullItemWrapper.uniqueId] = apullItemWrapper
                 }
             }

         }

     }

     fun getWorkingTemplate(downloadId: String?) : ApullTemplateBase?{
          return mWorkingTemplates[downloadId]?.getTemplate()
     }

    fun getRewardInfo(uniqueId:String?) : RewardUtils.RewardInfo?{
        if(!uniqueId.isNullOrEmpty()){
            return mRewardInfos[uniqueId]
        }
        return null
    }

    fun getTryResponse(uniqueId: String?):RewardTryResponse?{
        if(!uniqueId.isNullOrEmpty()){
            return mTryResponseMap[uniqueId]
        }
        return null
    }

    fun cacheTryResponse(uniqueId: String?,tryResponse: RewardTryResponse?){
        if(!uniqueId.isNullOrEmpty()) {
            if (!mTryResponseMap.containsKey(uniqueId)) {
                mTryResponseMap[uniqueId!!] = tryResponse
            }
        }
    }

    fun cacheRewardInfo(uniqueId: String?,rewardInfo: RewardUtils.RewardInfo?){
        if(!uniqueId.isNullOrEmpty()) {
            if (!mRewardInfos.containsKey(uniqueId) && rewardInfo != null) {
                mRewardInfos[uniqueId!!] = rewardInfo
            }
        }
    }

    fun cacheTwiceTemplates(list: MutableList<TemplateBase?>?){
        list?.apply {
            val lists = list.filter{
                it?.apullTemplateBase != null
            }.map {
                template -> template?.apullTemplateBase
            }
            if(lists.isNotEmpty()) {
                mTwiceDialogTemplates.addAll(lists)
            }
        }
    }

    fun getTwiceTemplates():MutableList<ApullTemplateBase?>? {
        return mTwiceDialogTemplates
    }

    fun clearTwiceTemplates(){
        mTwiceDialogTemplates.clear()
    }

    fun cacheHourTemplate(scene:Int,mSubScene:Int,templateBase: ApullTemplateBase?){
        templateBase?.apply {
            val key =  String.format("%08d%08d%08d", scene, mSubScene, 0)
            mRewardHourTemplates[key] = ApullItemWrapper(templateBase)
        }
    }

    fun getHourTemplate(scene:Int,mSubScene:Int):ApullTemplateBase?{
        val key =  String.format("%08d%08d%08d", scene, mSubScene, 0)
        return mRewardHourTemplates[key]?.getTemplate()
    }
}