package com.qihoo360.mobilesafe.reward.ui.page

import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.graphics.Color
import android.os.Build
import android.text.TextUtils
import android.view.View
import android.webkit.JavascriptInterface
import android.webkit.WebView
import android.widget.AdapterView
import android.widget.RelativeLayout
import com.qihoo360.apullsdk.apull.control.sync.ApullActionConst
import com.qihoo360.apullsdk.apull.protocol.model.ApullTemplateBase
import com.qihoo360.apullsdk.export.support.NewsExportArgsUtil
import com.qihoo360.apullsdk.utils.DensityUtil
import com.qihoo360.apullsdk.utils.JsonHelper
import com.qihoo360.apullsdk.utils.NetUtil
import com.qihoo360.apullsdk.utils.WebViewResourceHelper
import com.qihoo360.i.Factory
import com.qihoo360.i.IPluginManager
import com.qihoo360.mobilesafe.LocalBroadcastManagerC
import com.qihoo360.mobilesafe.RewardSDK
import com.qihoo360.mobilesafe.extension.setVisible
import com.qihoo360.mobilesafe.extension.tryCatch
import com.qihoo360.mobilesafe.reward.R
import com.qihoo360.mobilesafe.reward.RewardConstant
import com.qihoo360.mobilesafe.reward.data.ApullItemWrapper
import com.qihoo360.mobilesafe.reward.data.RewardUtils
import com.qihoo360.mobilesafe.reward.ui.page.dialog.RewardDownloadDialog
import com.qihoo360.mobilesafe.reward.ui.page.dialog.RewardDownloadHourDialog
import com.qihoo360.mobilesafe.reward.ui.page.dialog.RewardResultHourDialog
import com.qihoo360.mobilesafe.support.log.Logger
import com.qihoo360.mobilesafe.ui.common.other.CommonPopupWindow
import com.qihoo360.mobilesafe.ui.common.other.CommonUIUtils
import com.qihoo360.mobilesafe.ui.common.other.CommonWebChromeClientEx
import com.qihoo360.newssdk.apull.page.ApullAdWebViewPage
import kotlinx.android.synthetic.main.page_raffle_hour.*
import org.json.JSONObject

open class RaffleHourActivity : RaffleBaseActivity(){

    private val DEFAULT_POOL = "hourly"

    private var batteryQuantity = 0

    private var apullTemplate: ApullTemplateBase? = null

    private val TAG = "RaffleHourActivity"

    /**
     * 红包活动现在的状态
     * 1：正在进行
     * 2、已结束
     * 3、即将开始
     */
    private var mRaffleState: Int = 0

    override fun setLayoutId(): Int  = R.layout.page_raffle_hour

    private val mRewardReceiver = RewardReceiver()
    private val mProcessReceiver = ProcessReceiver()

    override fun initData() {
        super.initData()
        tryCatch {
            val mReceiverFilter = IntentFilter()
            mReceiverFilter.addAction(Intent.ACTION_BATTERY_CHANGED)
            mReceiverFilter.addAction(ApullActionConst.ACTION_QIHOO_NEWSDK_DOWNLOAD_ONSTARTINSTALLAPK)
            mReceiverFilter.addAction(ApullActionConst.ACTION_QIHOO_NEWSDK_DOWNLOAD_ONAPKINSTALLED)
            mReceiverFilter.addAction(RewardConstant.ACTION_APULL_APP_AD_CLICK)
            registerReceiver(mRewardReceiver,mReceiverFilter)

            val mProcessFilter = IntentFilter()
            mProcessFilter.addAction(RewardConstant.ACTION_ACTIVITY_CHANGE_EVENT)
            LocalBroadcastManagerC.getInstance(this).registerReceiver(mProcessReceiver,mProcessFilter)
        }
    }

    override fun initDefaultData() {
        mFromIntentData.apply {
            // 更新默认值
            if (TextUtils.isEmpty(mUrl)) {
                mUrl = "http://pop.shouji.360.cn/weishi/spring/pop.html"
            }
            if (TextUtils.isEmpty(mPromotionId)) {
                mPromotionId = "1"
            }
            if (TextUtils.isEmpty(mWxImgUrl)) {
                mWxImgUrl = "http://p0.qhimg.com/d/inn/106b6c8b/puq.png"
            }
            if (TextUtils.isEmpty(mWbImgUrl)) {
                mWbImgUrl = "http://p9.qhimg.com/d/inn/2a27ed0b/br.png"
            }
            if (TextUtils.isEmpty(mShareTitle)) {
                mShareTitle = getString(R.string.share_title_key)
            }
            if (TextUtils.isEmpty(mShareDesc)) {
                mShareDesc = getString(R.string.share_desc_key)
            }

            Logger.d(TAG,"mScene = $mScene,mSubScene = $mSubScene,mTitle=$mTitle,mUrl=$mUrl," +
                    "mUrl=$mUrl,mPromotionId=$mPromotionId,mBackground=$mBackground,mWxImgUrl=$mWxImgUrl," +
                    "mWbImgUrl = $mWbImgUrl,mShareTitle=$mShareTitle,mShareDesc=$mShareDesc")
        }
    }

    override fun initWebView() {
        super.initWebView()

        mWebView.setWebChromeClient(CommonWebChromeClientEx())
        if (!TextUtils.isEmpty(mFromIntentData.mUrl)) {
            mWebView.loadUrl(mFromIntentData.mUrl)
        }
        mWebView?.addJavascriptInterface(RaffleJavaScriptInterface(this), "raffle_interface")

        if (DEBUG && Build.VERSION.SDK_INT >= 19) {
            tryCatch {
                val m = WebView::class.java.getMethod("setWebContentsDebuggingEnabled", Boolean::class.javaPrimitiveType)
                m.invoke(null, true)
            }
        }
        tryCatch {
            WebViewResourceHelper.addChromeResourceIfNeeded(this)
        }
    }

    override fun onPageFinished(view: WebView?, url: String?) {
        mHongbaoTitle?.setTitle(view?.title)
    }

    internal inner class RaffleJavaScriptInterface(private val mContext: Context) {

        @JavascriptInterface
        fun getNetState(): String {
            Logger.d(TAG, "getNetState")
            val netType: String
            val connectionType = NetUtil.getConnectionType(mContext)
            netType = when (connectionType) {
                NetUtil.TYPE_2G -> "1"
                NetUtil.TYPE_3G -> "2"
                NetUtil.TYPE_4G -> "3"
                NetUtil.TYPE_WIFI -> "4"
                else -> "5"
            }
            Logger.d(TAG, "getNetState netType = $netType")
            return netType
        }

        @JavascriptInterface
        fun requestApull(jsonData: String) {
            tryCatch {
                Logger.d(TAG, "requestApull jsonData = $jsonData")
                val jsonObj = JSONObject(jsonData)
                val pageId = jsonObj.optInt("pageId")
                val subPageId = jsonObj.optInt("subPageId")
                val state = jsonObj.optInt("state")
                val pool = jsonObj.optString("pool")
                var useDefaultPool = mFromIntentData.mPool != pool
                if (DEBUG) {
                    useDefaultPool = true
                }

                if (mFromIntentData.mScene != pageId || mFromIntentData.mSubScene != subPageId || mRaffleState != state || useDefaultPool) {
                    mFromIntentData.mScene = pageId
                    mFromIntentData.mSubScene = subPageId
                    mRaffleState = state
                    mFromIntentData.mPool = pool
                    if (DEBUG) {
                        mFromIntentData.mPool = DEFAULT_POOL
                    }
                    mContainerViewStub?.setVisible(false)
                    mHongbaoFinish?.setVisible(false)
                    mHongbaoWillStart?.setVisible(false)

                    when (mRaffleState) {
                        1 -> {
                            // 在map中查找是否有数据后，再判断要不要请求数据
                            apullTemplate = mPresenter.getHourTemplate(pageId,subPageId)
                            if (apullTemplate != null) {
                                val container = mPresenter.buildSingleRewardContainer(this@RaffleHourActivity, apullTemplate)
                                invokeJSInterface(container)
                            } else {
                                // 请求广告
                                requestRewardAd()
                            }
                        }
                        2 -> mHongbaoFinish?.setVisible(true)
                        3 -> mHongbaoWillStart?.setVisible(true)
                    }
                }
            }

        }
    }

    override fun onClick(v: View?) {
        super.onClick(v)
        when (v?.id) {
            R.id.mHongbaoRetry -> {
                mHongbaoRetry?.setVisible(false)
                requestRewardAd()
            }

        }
    }

    override fun requestRewardAd() {
        super.requestRewardAd()
        if(NetUtil.isConnected(this@RaffleHourActivity)) {
            super.requestRewardAd()
            mPresenter.requestRewardAd(this@RaffleHourActivity, mFromIntentData.mScene, mFromIntentData.mSubScene,
                    mFromIntentData.mReferScene, mFromIntentData.mReferSubScene)
        }
    }

    override fun onRewardTimeOut() {
        super.onRewardTimeOut()
        mHongbaoRetry?.setVisible(true)
    }

    override fun onRewardResponse(view: View?, pageCount: Int) {
        super.onRewardResponse(view, pageCount)
        if(view != null){
            invokeJSInterface(view)
        }
    }

    fun invokeJSInterface(view :View?){
        // 调用 js 接口
        apullTemplate = mPresenter.getHourTemplate(mFromIntentData.mScene,mFromIntentData.mSubScene)
        apullTemplate?.apply {

            val jo = JSONObject()
            var apullItemWrapper = ApullItemWrapper(apullTemplate!!)
            JsonHelper.putStringJo(jo, "name", apullItemWrapper.getAppName())
            JsonHelper.putStringJo(jo, "icon", apullItemWrapper.getIconUrl())
            JsonHelper.putStringJo(jo, "package_name", apullItemWrapper.getRewardPackageName())
            var jsonData = jo.toString()
            var i = 0
            while (i < jsonData.length) {
                if (jsonData[i] == '"') {
                    jsonData = jsonData.substring(0, i) + "\\" + jsonData.substring(i)
                    i++
                }
                i++
            }
            val url = "javascript:setPageInfo('$jsonData')"
            Logger.d(TAG, "js : $url")
            mWebView.loadUrl(url)

            if (mContainerViewStub != null) {
                mContainerViewStub.removeAllViews()
                mContainerViewStub.addView(view)
                mContainerViewStub.visibility = View.VISIBLE
            }
        }?:kotlin.run {
            mHongbaoRetry?.setVisible(true)
        }
    }

    override fun initTitleView() {
        super.initTitleView()
        mHongbaoTitle.height = 56
        mHongbaoTitle.setBackgroundColor(Color.parseColor("#00000000"))
        mHongbaoTitle.hideDivider()
        mHongbaoTitle.setSettingImg(R.drawable.setting_button_image)
        mHongbaoTitle.setOnSettingListener {
            mPopupWindow?.apply {
                tryCatch({

                }) {
                    mPopupWindow!!.showAsDropDown(mHongbaoTitle.rightButton, -CommonUIUtils.dip2px(this@RaffleHourActivity, 120f), 0)
                }
            }?:kotlin.run {
                mPopupWindow = CommonPopupWindow(this@RaffleHourActivity)
                mPopupWindow?.setCheckVisible(false)
                mPopupWindow?.setWindowWidth(CommonUIUtils.dip2px(this@RaffleHourActivity, 150f))
                mPopupWindow?.setOnItemClickListener(ItemClickListener())
                val items = resources.getStringArray(R.array.hongbao_titlebar_array_hour)
                mPopupWindow?.setItems(items)
                tryCatch {
                    mPopupWindow?.showAsDropDown(mHongbaoTitle.rightButton, -CommonUIUtils.dip2px(this@RaffleHourActivity, 120f), 0)
                }
            }
        }
        mHongbaoTitle.setShareImg(R.drawable.hongbao_record)
        mHongbaoTitle.setOnShareListener {
            mPresenter.showRewardRecord()
        }

        mHongbaoRetry.setOnClickListener(this)

        if (isVirtualKeyShow(this)) {

            var param: RelativeLayout.LayoutParams? = mContainerViewStub?.layoutParams as? RelativeLayout.LayoutParams
            param?.bottomMargin = DensityUtil.dip2px(this, 28f)
            param = mHongbaoRetry?.layoutParams as? RelativeLayout.LayoutParams
            param?.bottomMargin = DensityUtil.dip2px(this, 28f)
            param = mHongbaoFinish?.layoutParams as? RelativeLayout.LayoutParams
            param?.bottomMargin = DensityUtil.dip2px(this, 28f)
            param = mHongbaoWillStart?.layoutParams as? RelativeLayout.LayoutParams
            param?.bottomMargin = DensityUtil.dip2px(this, 28f)
        }
    }


    private fun isVirtualKeyShow(context: Activity): Boolean {
        val resources = context.resources
        val resourceId = resources.getIdentifier("config_showNavigationBar", "bool", "android")
        return if (resourceId > 0) {
            resources.getBoolean(resourceId)
        } else false
    }

    private inner class ItemClickListener : AdapterView.OnItemClickListener {
        override fun onItemClick(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
            when(position){
                0->{
                    Factory.startActivity(view?.context, Intent(), "wallet", "com.qihoo360.mobilesafe.wallet.MainActivity", IPluginManager.PROCESS_AUTO)
                }
                1->{
                    val i = Intent()
                    i.putExtra(NewsExportArgsUtil.KEY_INITIAL_URL, "http://pop.shouji.360.cn/weishi/pop/rulered.html")
                    if(RewardSDK.mIsSingleApk){
                        i.setClass(view?.context,ApullAdWebViewPage::class.java)
                        view?.context?.startActivity(i)
                        return
                    }
                    Factory.startActivity(view?.context, i, "news", "com.qihoo360.newssdk.apull.page.ApullAdWebViewPage", IPluginManager.PROCESS_AUTO)
                }
            }
        }

    }

    private inner class RewardReceiver : BroadcastReceiver(){
        override fun onReceive(context: Context?, intent: Intent?) {

            intent?.run {
                Logger.d(TAG, "RewardReceiver: : $action")
                when(action){
                    ApullActionConst.ACTION_QIHOO_NEWSDK_DOWNLOAD_ONSTARTINSTALLAPK ->{
                        val downloadId = getStringExtra(ApullActionConst.KEY_RESULT_DOWNLOAD_ID)
                        mPresenter.handleWorkingPackage(downloadId)
                    }
                    ApullActionConst.ACTION_QIHOO_NEWSDK_DOWNLOAD_ONAPKINSTALLED ->{
                        val downloadId = getStringExtra(ApullActionConst.KEY_RESULT_DOWNLOAD_ID)
                        mFromIntentData.mBatteryQuantity = batteryQuantity
                        mPresenter.handlePackageInstalled(downloadId,mFromIntentData)
                    }
                    Intent.ACTION_BATTERY_CHANGED ->{
                        val level = getIntExtra("level",0)
                        val scale = getIntExtra("scale",100)
                        batteryQuantity = level * 100 / scale
                    }
                    RewardConstant.ACTION_APULL_APP_AD_CLICK ->{
                        val uniqueId = getStringExtra(RewardConstant.REWARD_AD_UNIQUE_ID)
                        mPresenter.handleWorkingPackage(uniqueId)
                        mFromIntentData.mBatteryQuantity = batteryQuantity
                        mPresenter.handlePackageInstalled(uniqueId,mFromIntentData)
                    }
                }
            }
        }

    }

    private inner class ProcessReceiver : BroadcastReceiver(){
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.run {
                Logger.d(TAG, "ProcessReceiver: : $action")
                when(action){
                    RewardConstant.ACTION_ACTIVITY_CHANGE_EVENT ->{
                        val pkgName = getStringExtra("packageName")
                        mPresenter.computeAppActiveTime(pkgName)
                    }
                }
            }
        }

    }

    override fun showRewardDownloadDialog(apullItemWrapper: ApullItemWrapper,
                                          mDownloadListener: RewardDownloadDialog.DownloadListener) {
        tryCatch({
            Logger.d(TAG,"showRewardDownloadDialog")
            Logger.handleException(it)
        }){
            mRewardDownloadDialog?.apply {
                if(!apullItemWrapper.equalsUniqueId(mRewardDownloadDialog?.getApullItemWrapper())){
                    mRewardDownloadDialog = RewardDownloadHourDialog(this@RaffleHourActivity,apullItemWrapper,mDownloadListener)
                }
                if(mDownloadListener != null) {
                    mRewardDownloadDialog?.setDownloadListener(mDownloadListener)
                }
                if(!isShowing){
                    show()
                }
            }?:run{
                mRewardDownloadDialog = RewardDownloadHourDialog(this@RaffleHourActivity,apullItemWrapper,mDownloadListener)
                mRewardDownloadDialog?.show()
            }
        }

    }

    override fun showChanceResultDownloadDialog(rewardInfo : RewardUtils.RewardInfo?) {
        tryCatch({
            Logger.d(TAG,"updateRewardDownloadDialogStatus")
            Logger.handleException(it)
        }){
            mRewardDownloadDialog?.apply {
                if(rewardInfo?.apullAdItem != null) {
                    if (!rewardInfo.apullAdItem.equalsUniqueId(mRewardDownloadDialog?.getApullItemWrapper())) {
                        mRewardDownloadDialog = RewardDownloadHourDialog(this@RaffleHourActivity, rewardInfo.apullAdItem, null)
                    }
                }
                if(!isShowing) {
                    show()
                }
                updateSuccessRewardInfo(rewardInfo)

            } ?: run{
                mRewardDownloadDialog = RewardDownloadHourDialog(this@RaffleHourActivity, rewardInfo?.apullAdItem, null)
                mRewardDownloadDialog?.show()
                mRewardDownloadDialog?.updateSuccessRewardInfo(rewardInfo)
            }
        }
    }

    override fun showFailResultDownloadDialog(rewardInfo: RewardUtils.RewardInfo?, status: Int, rewardType: Int) {
        tryCatch({
            Logger.d(TAG,"updateRewardDownloadDialogStatus")
            Logger.handleException(it)
        }){
            mRewardDownloadDialog?.apply {
                if(rewardInfo?.apullAdItem != null) {
                    if (!rewardInfo.apullAdItem.equalsUniqueId(mRewardDownloadDialog?.getApullItemWrapper())) {
                        mRewardDownloadDialog = RewardDownloadHourDialog(this@RaffleHourActivity, rewardInfo.apullAdItem, null)
                    }
                }
                if(!isShowing) {
                    show()
                }
                updateFailRewardInfo(rewardInfo,status,rewardType)

            } ?: run{
                mRewardDownloadDialog = RewardDownloadHourDialog(this@RaffleHourActivity, rewardInfo?.apullAdItem, null)
                mRewardDownloadDialog?.show()
                mRewardDownloadDialog?.updateFailRewardInfo(rewardInfo,status,rewardType)
            }
        }
    }

    override fun showRewardResultDialog(rewardInfo: RewardUtils.RewardInfo?,succeed:Boolean,shareCanTryAgain: Boolean) {
        tryCatch({
            Logger.d(TAG,"handleRewardCommit")
            Logger.handleException(it)
        }){
            mRewardResultDialog?.apply {
                mRewardResultDialog?.updateRewardInfo(rewardInfo,succeed,shareCanTryAgain)
                if(!isShowing){
                    show()
                }
            }?:kotlin.run {
                mRewardResultDialog = RewardResultHourDialog(this,rewardInfo,succeed,shareCanTryAgain)
                mRewardResultDialog?.show()
            }
        }
    }


    override fun startRewardCommit(rewardInfo: RewardUtils.RewardInfo?) {
        tryCatch({
            Logger.d(TAG,"handleRewardCommit")
            Logger.handleException(it)
        }){
            mPresenter.startRewardCommit(rewardInfo)
        }
    }

    override fun dismissRewardDownloadDialog() {
        tryCatch{
            mRewardDownloadDialog?.apply {
                if(mRewardDownloadDialog?.isShowing == true) {
                    mRewardDownloadDialog?.dismiss()
                }
                mRewardDownloadDialog = null
            }
            mRewardResultDialog?.apply {
                if(mRewardResultDialog?.isShowing == true) {
                    mRewardResultDialog?.dismiss()
                }
                mRewardResultDialog = null
            }

        }
    }

    override fun onBackPressedDownDialog(rewardInfo: RewardUtils.RewardInfo?) {
        tryCatch {
            mPresenter.onBackPressedDownDialog(rewardInfo)
            mRewardDownloadDialog = null
            dismissRewardDownloadDialog()
        }
    }


    override fun onResume() {
        super.onResume()
        mPresenter.startCheckRewardChanceOnResume()
    }

    override fun onDestroy() {
        super.onDestroy()
        tryCatch {
            LocalBroadcastManagerC.getInstance(this).unregisterReceiver(mProcessReceiver)
            unregisterReceiver(mRewardReceiver)
            mPresenter.onDestroy()
        }
    }

}