package com.qihoo360.mobilesafe.reward.utils


import com.qihoo360.mobilesafe.extension.tryCatch
import com.qihoo360.mobilesafe.reward.data.ApullItemWrapper
import com.qihoo360.mobilesafe.reward.data.RewardUtils
import com.qihoo360.mobilesafe.support.log.Logger

import java.lang.ref.WeakReference
import java.util.ArrayList
import java.util.Collections

object TryResponseListenerManager {
    private val sMap = Collections.synchronizedMap(
            HashMap<String,WeakReference<OnTryResponseListener>>())

    fun register(uniqueId:String?,listener: OnTryResponseListener?) {
        tryCatch {
            if (uniqueId.isNullOrEmpty() || listener == null) {
                return
            }

            for((key,value) in sMap){
                if(value?.get() != null){
                    if(key == uniqueId && listener == value.get()){
                        return@tryCatch
                    }
                }
            }
            sMap[uniqueId] = WeakReference(listener)
        }
    }

    fun clearAll() {
        tryCatch {
            sMap?.clear()
        }
    }

    fun unRegister(uniqueId: String?,listener: OnTryResponseListener?) {
        synchronized(TryResponseListenerManager::class.java) {
            val iterator = sMap.keys.iterator()
            while (iterator.hasNext()) {
                val uniqueIds = iterator.next()
                val weak = sMap[uniqueIds]
                if (uniqueId == uniqueIds && listener == weak?.get()) {
                    iterator.remove()
                }
            }
        }
    }

    fun notifyOnTryResponseChanged(info: RewardUtils.RewardInfo?) {
        synchronized(TryResponseListenerManager::class.java) {
            val iterator = sMap.keys.iterator()
            while (iterator.hasNext()) {
                val uniqueIds  = iterator.next()
                val weak = sMap[uniqueIds]
                if(info?.apullAdItem?.uniqueId == uniqueIds) {
                    if (weak?.get() != null) {
                        val listener = weak.get()
                        listener?.onTryResponseCallBack(info)
                    } else {
                        iterator.remove()
                    }
                }
            }
        }
    }

}
